'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.HttpCoordinator = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _lodash = require('lodash.defaults');

var _lodash2 = _interopRequireDefault(_lodash);

var _requestPromiseAny = require('request-promise-any');

var _requestPromiseAny2 = _interopRequireDefault(_requestPromiseAny);

var _biguintFormat = require('biguint-format');

var _biguintFormat2 = _interopRequireDefault(_biguintFormat);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _anyPromise = require('any-promise');

var _anyPromise2 = _interopRequireDefault(_anyPromise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var HttpCoordinator = function HttpCoordinator(options) {
  var _this = this;

  var dateInstance = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

  this.lastHeartbeatTs = null;
  this.coordinatorId = this.generateNonce(8);
  this.coordination = null;
  this.options = (0, _assign2.default)({}, options);
  (0, _lodash2.default)(this.options, {
    appId: null,
    privateKeyFile: null,
    heartbeatTtl: 300000, // 5min in ms
    coordinatorUrl: null,
    datacenterId: null,
    workerIdMask: null,
    machineIdMask: null
  });
  if (!this.options.coordinatorUrl) {
    throw new Error('Missing `coordinatorUrl`');
  }
  if (!this.options.appId) {
    throw new Error('Missing `appId`');
  }
  if (!this.options.privateKeyFile) {
    throw new Error('Missing `privateKeyFile`');
  }
  this.privateKey = _fs2.default.readFileSync(this.options.privateKeyFile);

  this.dateInstance = dateInstance || Date;
  this.onChangeCallback = null;

  this.coordinate = function (onChangeCallback) {
    var onErrorCallback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var timeout = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 15000;

    _this.onChangeCallback = onChangeCallback;
    _this.onErrorCallback = onErrorCallback;
    return _this.tryHeartbeat().timeout(timeout, 'HttpCoordinator timed out: ' + timeout + 'ms').catch(function (err) {
      console.error(err);
      if (_this.onErrorCallback) {
        _this.onErrorCallback(err);
      }
    });
  };

  this.postToHttpCoordinator = function () {
    var nonce = _this.generateNonce();
    var rpOptions = {
      method: 'POST',
      uri: _this.options.coordinatorUrl,
      body: {
        coordinatorId: _this.coordinatorId,
        options: {
          appId: _this.options.appId,
          heartbeatTtl: _this.options.heartbeatTtl
        },
        request: {
          nonce: nonce,
          ts: _this.dateInstance.now()
        }
      },
      json: true
    };
    if (_this.options.datacenterId) {
      rpOptions.body.options.datacenterId = _this.options.datacenterId;
    }
    if (_this.options.workerIdMask) {
      rpOptions.body.options.workerIdMask = _this.options.workerIdMask;
    }
    if (_this.options.machineIdMask) {
      rpOptions.body.options.machineIdMask = _this.options.machineIdMask;
    }
    if (_this.coordination) {
      rpOptions.body.request.coordination = _this.coordination;
    }
    rpOptions.body.signature = _this.signRequest(rpOptions.body.request);

    return (0, _requestPromiseAny2.default)(rpOptions).then(function (parsedBody) {
      return _this.parseCoordinationResponse(parsedBody);
    });
  };

  this.shouldRunHeartbeat = function () {
    return _this.dateInstance.now() - _this.lastHeartbeatTs >= _this.options.heartbeatTtl;
  };

  this.tryHeartbeat = function () {
    if (_this.shouldRunHeartbeat()) {
      return _this.runHeartbeat();
    }
    return _anyPromise2.default.resolve(true);
  };

  this.runHeartbeat = function () {
    // do this so we avoid multiple attempts
    _this.lastHeartbeatTs = _this.dateInstance.now();
    return _this.postToHttpCoordinator().then(function (coordination) {
      _this.coordination = coordination;
      // then set it here again for a true idea of when we should wait
      _this.lastHeartbeatTs = _this.dateInstance.now();
      process.nextTick(function () {
        _this.onChangeCallback(null, _this.coordination);
      });
      return true;
    });
  };
};

HttpCoordinator.prototype.parseCoordinationResponse = function (coordination) {
  // TODO - reserved in case we need to finagle with the response
  return coordination;
};

HttpCoordinator.prototype.signRequest = function (request) {
  var cryptoSign = _crypto2.default.createSign('sha256');
  cryptoSign.update((0, _stringify2.default)(request));

  return cryptoSign.sign(this.privateKey).toString('hex');
};

/**
 * Generate a 128-bit nonce for the coordination request
 */
HttpCoordinator.prototype.generateNonce = function () {
  var sizeBytes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 16;

  var buffer = new Buffer(sizeBytes);
  buffer.fill(0);
  for (var i = 0; i < sizeBytes; i++) {
    buffer.writeUInt8(Math.floor(Math.random() * 256), i);
  }

  return (0, _biguintFormat2.default)(buffer, 'hex');
};

exports.default = HttpCoordinator;
exports.HttpCoordinator = HttpCoordinator;
//# sourceMappingURL=HttpCoordinator.js.map